#!/bin/bash

set -e

. /usr/pluto/bin/Network_Parameters.sh 2>/dev/null 
. /usr/pluto/bin/Config_Ops.sh 2>/dev/null
. /usr/pluto/bin/Section_Ops.sh 2>/dev/null
. /usr/pluto/bin/LockUtils.sh 2>/dev/null
. /usr/pluto/bin/Utils.sh 2>/dev/null

DEVICEDATA_Extra_Parameters=139
DEVICEDATA_Extra_Parameters_Override=140
DEVICEDATA_Architecture=112
DEVICEDATA_DisklessBoot=9
DEVICEDATA_DisklessImages=258
DEVICEDATA_Model=233

DEVICEDATA_PK_Distro=7

function setup_tftp_boot 
{
	echo "* Configuring TFTP Boot for MD #${Moon_DeviceID}"

	local Moon_BootConfFile="/tftpboot/pxelinux.cfg/01-$(echo ${Moon_MAC//:/-} | tr 'A-Z' 'a-z')"
	local BootConf=""

	local BootParams="quiet splash video=uvesafb:mode_option=1024x768-24,mtrr=3,scroll=ywrap"
	local BootParams_Extra=$(RunSQL "SELECT IK_DeviceData FROM Device_DeviceData WHERE FK_Device = $Moon_DeviceID AND FK_DeviceData = $DEVICEDATA_Extra_Parameters")
	local BootParams_Override=$(RunSQL "SELECT IK_DeviceData FROM Device_DeviceData WHERE FK_Device = $Moon_DeviceID AND FK_DeviceData = $DEVICEDATA_Extra_Parameters_Override")
	if [[ "$BootParams_Override" == "1" ]] ;then
		BootParams="${BootParams_Extra}"
	else
		BootParams="${BootParams} ${BootParams_Extra}"
	fi

	local DefaultBootName=
	
	mkdir -p /tftpboot/pxelinux.cfg

	if [[ -z "$Moon_DisklessImages" ]]; then
		BootConf="${BootConf}DEFAULT Pluto\n"
		BootConf="${BootConf}LABEL Pluto\n"
		BootConf="${BootConf}KERNEL ${Moon_DeviceID}/vmlinuz\n"
		BootConf="${BootConf}APPEND initrd=${Moon_DeviceID}/initrd.img ramdisk=10240 rw root=/dev/nfs boot=nfs nfsroot=${IntIP}:/usr/pluto/diskless/${Moon_DeviceID} ${BootParams}\n"
		
		echo -e "$BootConf" > "$Moon_BootConfFile"

		#mkdir -p /tftpboot/${Moon_DeviceID}
		#rm -f /tftpboot/${Moon_DeviceID}/vmlinuz
		#ln -s ${Moon_RootLocation}/boot/vmlinuz /tftpboot/${Moon_DeviceID}/vmlinuz
		#rm -f /tftpboot/${Moon_DeviceID}/initrd.img
		#ln -s ${Moon_RootLocation}/boot/initrd.img /tftpboot/${Moon_DeviceID}/initrd.img
      		
		## Find kernel and initrd
		Kernel=
		Initrd=
		for File in /usr/pluto/diskless/"$Moon_DeviceID"/boot/vmlinuz-*; do
			Kernel="$File"
		done
		if [[ -z "$Kernel" ]]; then
			echo "WARNING: Missing kernel or initrd file. Cannot set for PXE boot."
			continue
		fi
		Kver=$(basename "$Kernel")
		Kver="${Kver#vmlinuz-}"
		Initrd=/usr/pluto/diskless/"$Moon_DeviceID"/boot/initrd.img-"$Kver"

		if [[ -z "$Kernel" || ! -f "$Kernel" || -z "$Initrd" || ! -f "$Initrd" ]]; then
			echo "WARNING: Missing kernel or initrd file. Cannot set for PXE boot."
			continue
		fi

        	mkdir -p /tftpboot/${Moon_DeviceID}
        	ln -sf "$Kernel" /tftpboot/${Moon_DeviceID}/vmlinuz
        	ln -sf "$Initrd" /tftpboot/${Moon_DeviceID}/initrd.img

		## these are needed because by default they see the kernel running on the core,
		## which may be different from the one installed on the MD, thus not doing what we want by default
		chroot /usr/pluto/diskless/"$Moon_DeviceID" depmod "$Kver"
		chroot /usr/pluto/diskless/"$Moon_DeviceID" update-initramfs -k "$Kver" -u
	else
		for line in $Moon_DisklessImages; do
			NameKernel="${line%%=*}"
			if [[ "$NameKernel" == *-* ]]; then
				Name="${NameKernel%%-*}"
				Kernel="${NameKernel#*-}"
			else
				Name="$NameKernel"
				Kernel=
			fi
			[[ -z "$Kernel" ]] && continue || :
			[[ -z "$DefaultBootName" ]] && DefaultBootName="$Name" || :
			Value="${line#*=}"
			mkdir -p "/tftpboot/${Moon_DeviceID}/$Name"
			rm -f "/tftpboot/${Moon_DeviceID}/$Name/"{vmlinuz,initrd.img}
                        if [[ ! -f "${Moon_RootLocation}/boot/vmlinuz-${Kernel}" ]]; then
                        	# If the moon's kernel image does not correspond to the core's image, we have a problem
                                # and we try to install the correct version
                                chroot "${Moon_RootLocation}" apt-get install "linux-image-${Kernel}"
                        fi
                                                                                                                                                
                        ln -s "${Moon_RootLocation}/boot/vmlinuz-${Kernel}" "/tftpboot/${Moon_DeviceID}/$Name/vmlinuz"
			ln -s "${Moon_RootLocation}/boot/initrd.img-${Kernel}" "/tftpboot/${Moon_DeviceID}/$Name/initrd.img"
		done

		BootConf="${BootConf}DEFAULT Pluto\n"
		BootConf="${BootConf}LABEL Pluto\n"
		BootConf="${BootConf}KERNEL ${Moon_DeviceID}/$DefaultBootName/vmlinuz\n"
		BootConf="${BootConf}APPEND initrd=${Moon_DeviceID}/$DefaultBootName/initrd.img ramdisk=10240 rw root=/dev/nfs boot=nfs nfsroot=${IntIP}:/usr/pluto/diskless/${Moon_DeviceID}/$DefaultBootName ${BootParams}\n"
		
		echo -e "$BootConf" > "$Moon_BootConfFile"
	fi
}

function setup_mysql_access 
{
	echo "* Setting up MySQL access for MD #${Moon_DeviceID}"
	RunSQL "GRANT ALL PRIVILEGES ON *.* TO '$MySqlUser'@$Moon_IP; GRANT ALL PRIVILEGES ON *.* TO 'eib'@$Moon_IP"
	RunSQL "FLUSH PRIVILEGES"
}


function generate_diskless_installer 
{
	
	echo "* Generating Pluto Installer for MD #${Moon_DeviceID}"

	## Copy installer files
	mkdir -p $Moon_RootLocation/usr/pluto/install
	Files="Common.sh ConfirmDependencies_Debian.sh AptSources.sh"
	for Stuff in $Files; do
		cp /usr/pluto/install/$Stuff $Moon_RootLocation/usr/pluto/install
	done

	Moon_DistroID=$(RunSQL "SELECT IK_DeviceData FROM Device_DeviceData WHERE FK_Device='$Moon_DeviceID' AND FK_DeviceData='$DEVICEDATA_PK_Distro'")

	## Generate another installer script
	if [[ ! -f $Moon_RootLocation/usr/pluto/install/activation.sh ]]; then
		/usr/pluto/bin/ConfirmDependencies -o "$Moon_DistroID" -r -D "$MySqlDBName" $PLUTO_DB_CRED -d "$Moon_DeviceID" install > $Moon_RootLocation/usr/pluto/install/activation.sh
	fi
	
	## Modify a install script to run as for diskless
	sed '/^Type=/ s/^.*$/Type="diskless"/' /usr/pluto/install/Initial_Config.sh >$Moon_RootLocation/usr/pluto/install/Initial_Config.sh
	chmod +x $Moon_RootLocation/usr/pluto/install/Initial_Config.sh
	mkdir -p $Moon_RootLocation/usr/pluto/deb-cache
}


function setup_hosts_file 
{
	echo "* Setting up /etc/hosts"
	local Content=""
	local Q="
		SELECT 
			PK_Device, 
			IPaddress
		FROM 
			Device
			JOIN Device_DeviceData ON PK_Device = FK_Device AND FK_DeviceData = $DEVICEDATA_DisklessBoot
			JOIN DeviceTemplate ON FK_DeviceTemplate = PK_DeviceTemplate
		WHERE 
			FK_DeviceCategory = '8'
			AND
			FK_Device_ControlledVia IS NULL
			AND
			IK_DeviceData = '1'
	"
	local R=$(RunSQL "$Q")
	local Row
	for Row in $R ;do
		local DeviceID=$(Field 1 "$Row")
		local IP=$(Field 2 "$Row")

		if [[ "$IP" == "" ]] ;then
			continue
		fi

		Content="${Content}${IP}		moon${DeviceID}\n"
	done
	
	PopulateSection "/etc/hosts" "DisklessMD" "$Content"

	## Export hosts file to other computer
	echo | /usr/lib/yp/ypinit -m

	echo "---$Content---"
}


function update_config_files
{
	local ScriptDir="/usr/pluto/bin/files.d"
	local ScriptsList="cron.d-synctime fstab-diskless interfaces mythtv-mysql.txt nis-client pluto.conf resolv.conf syslog.conf timezone mountnfs.sh event.d apt.conf hostname mythtv-fix firstrun"
	local -a Names
	local NameKernel Name Kernel
	if [[ -n "$Moon_DisklessImages" ]]; then
		for line in $Moon_DisklessImages; do
			NameKernel="${line%%=*}"
			if [[ "$NameKernel" == *-* ]]; then
				Name="${NameKernel%%-*}"
				Kernel="${NameKernel#*-}"
			else
				Name="$NameKernel"
				Kernel=
			fi
			[[ -z "$Kernel" ]] && continue || :
			Names=("${Names[@]}" "$Name")
		done
	fi
	for Script in $ScriptsList ;do
		if [[ ! -x $ScriptDir/$Script ]] ;then
			echo "WARNING: Script $Script cannot be executed"
			continue
		fi

		pushd $ScriptDir
		if [[ "${#Names[@]}" -eq 0 ]]; then
			$ScriptDir/$Script --root "$Moon_RootLocation" --device "$Moon_DeviceID" --template "$Moon_DeviceTemplate"
		else
			for Name in "${Names[@]}"; do
				$ScriptDir/$Script --root "$Moon_RootLocation/$Name" --device "$Moon_DeviceID" --subname "$Name" --template "$Moon_DeviceTemplate"
			done
		fi
		popd
	done
}


## Parse parameters
SkipLock=n
for ((i = 1; i <= "$#"; i++)); do
        Parm="${!i}"
        case "$Parm" in                 --skiplock) SkipLock=y
        esac
done


## Aquire Lock
if [[ "$SkipLock" == n ]]; then
        if ! TryLock "Diskless_Setup" "Diskless_Setup"; then
                echo "Another Diskless-related operation is running"
                exit 1
        fi
        trap 'Unlock "Diskless_Setup" "Diskless_Setup"' EXIT
fi


## Check for valid DHCP settings else exit
if [ -z "$DHCPsetting" ]; then
	echo "Diskless MDs can't exist in the current setup (no DHCP). Not setting up any."
	exit
fi


## Getting a list of Media Directors
echo "Getting list of Media Directors"
Q="
	SELECT 
		PK_Device, 
		IPaddress, 
		MACaddress, 
		Device.Description,
		Device.NeedConfigure,
		Device.FK_DeviceTemplate
	FROM 
		Device
		JOIN Device_DeviceData ON PK_Device = FK_Device AND FK_DeviceData = $DEVICEDATA_DisklessBoot
		JOIN DeviceTemplate ON FK_DeviceTemplate = PK_DeviceTemplate
	WHERE 
		FK_DeviceCategory = '8'
		AND
		FK_Device_ControlledVia IS NULL
		AND
		IK_DeviceData = '1'
"
R=$(RunSQL "$Q")

## Processing Moons
for Row in $R; do
	## Gathering information about this moon
	Moon_DeviceID=$(Field 1 "$Row")

	Moon_IP=$(Field 2 "$Row")
	Moon_IP=$(/usr/pluto/bin/PlutoDHCP.sh -d "$Moon_DeviceID" -a)
	if [[ -z "$Moon_IP" ]]; then
		echo "WARNING : No free IP left to assign for moon$Moon_DeviceID"
		continue
	fi
	
	Moon_MAC=$(Field 3 "$Row")
	Moon_MAC=$(echo ${Moon_MAC//-/:} | tr 'a-z' 'A-Z')
	if ! /usr/pluto/bin/CheckMAC.sh "$Moon_MAC" ;then
		echo "WARNING : Mac ($Moon_MAC) of moon$Moon_DeviceID is invalid"
		continue
	fi

	Moon_Description=$(Field 4 "$Row")
	Moon_NeedConfigure=$(Field 5 "$Row")
	if [[ "$Moon_NeedConfigure" != "1" ]] ;then
		echo "INFO : Skiping moon$Moon_DeviceID because NeedConfigure flag is not set"
		continue
	fi

	Moon_DeviceTemplate=$(Field 6 "$Row")

	Moon_Architecture=$(RunSQL "SELECT IK_DeviceData FROM Device_DeviceData WHERE FK_Device='$Moon_DeviceID' AND FK_DeviceData='$DEVICEDATA_Architecture'")
	if [[ "$Moon_Architecture" == "" ]] ;then
		Moon_Architecture=i386
	fi

	Moon_Model=$(RunSQL "SELECT IK_DeviceData FROM Device_DeviceData WHERE FK_Device='$Moon_DeviceID' AND FK_DeviceData='$DEVICEDATA_Model'")
	Moon_Model="${Moon_Model%_*}_${Moon_Architecture}"
	RunSQL "UPDATE Device_DeviceData SET IK_DeviceData='$Moon_Model' WHERE FK_Device='$Moon_DeviceID' AND FK_DeviceData='$DEVICEDATA_Model'"


	Moon_RootLocation="/usr/pluto/diskless/$Moon_DeviceID"
	Moon_BootConfFile="/tftpboot/pxelinux.cfg/01-$(echo ${Moon_MAC//:/-} | tr 'A-Z' 'a-z')"
	Moon_DisklessImages=$(GetDeviceData "$Moon_DeviceID" "$DEVICEDATA_DisklessImages")


	
	## Adding moon to hosts (/etc/hosts)
	hosts_DisklessMD="${hosts_DisklessMD}${Moon_IP}	moon${Moon_DeviceID}\n"

	## Create the a filesystem for this MD
	/usr/pluto/bin/Diskless_CreateFS.sh "$Moon_DeviceID"
	/usr/pluto/bin/Diskless_InstallKernel.sh "$Moon_DeviceID"

	## Setting Up
	update_config_files
	setup_tftp_boot
	setup_mysql_access
	generate_diskless_installer

#	## Create /var/log/pluto for this device as a symlink
#	mkdir -p "/home/logs/diskless_${Moon_DeviceID}"
#	if [[ -d $Moon_RootLocation/var/log/pluto	]] ;then
#		mv $Moon_RootLocation/var/log/pluto/* /home/log/diskless_$Moon_DeviceID
#		rm -rf $Moon_RootLocation/var/log/pluto/
#	fi
#	if [[ ! -e "$Moon_RootLocation"/var/log/pluto ]]; then
#		ln -s "/home/logs/diskless_$Moon_DeviceID" "$Moon_RootLocation/var/log/pluto"
#	fi

	## Dome configuring this MD
	RunSQL "UPDATE Device SET NeedConfigure = 0 WHERE PK_Device=$Moon_DeviceID"
done

setup_hosts_file
/usr/pluto/bin/Update_StartupScrips.sh
/usr/pluto/bin/Diskless_ExportsNFS.sh
/usr/pluto/bin/sync_pluto2amp.pl

## Add host to the list of hosts allowed to access /home samba share
R=$(RunSQL "SELECT IPaddress FROM Device JOIN DeviceTemplate ON FK_DeviceTemplate = PK_DeviceTemplate WHERE FK_DeviceCategory = 8")
MoonIPs=""
for Row in $R; do
	IPaddress=$(Field 1 "$Row")
	if [[ "$IPaddress" == "" ]] ;then
		continue;
	fi

	MoonIPs="${MoonIPs} $IPaddress"
done

if [[ $(CheckSectionExists "/etc/samba/smb.conf" "Home Hosts Allow") == "true" ]] ;then
	PopulateSection "/etc/samba/smb.conf" "Home Hosts Allow" "hosts allow = $MoonIPs"
fi

## Do the update fix for current debian computers
for dir in /usr/pluto/diskless/* ;do
	if [[ -f "$dir/debian/etc/init.d/fastboot/rcS" ]] ;then
		sed -i 's/ApplyUpdates.sh/LMCEUpdate_Apply.sh/g' "$dir/debian/etc/init.d/fastboot/rcS" || :
		rm -f "$dir/debian/etc/rc2.d/S98LMCEUpdate" || :
	fi
done

echo "Finished setting up network boot for media directors."
echo "If new media director(s) were added, do a quick reload router."

